package org.simplericity.jettyconsole.jsp;

import org.apache.jasper.EmbeddedServletOptions;
import org.apache.jasper.servlet.JspServlet;
import org.eclipse.jetty.webapp.AbstractConfiguration;
import org.eclipse.jetty.webapp.WebAppContext;

import java.io.File;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

/**
 * Set TLD jars which were found when packaging the war.
 *
 * Configure Jasper's sysClassPath to only include Jasper/JSP/Servlet related jars
 *
 */
public class EmbeddedJspConfiguration extends AbstractConfiguration {

    @Override
    public void postConfigure(WebAppContext context) throws Exception {

        rewriteJasperSystemClasspath(context);

    }

    private void rewriteJasperSystemClasspath(WebAppContext context) throws NoSuchFieldException, IllegalAccessException {
        JspServlet jspServlet = (JspServlet) context.getServletHandler().getServlet("jsp").getServletInstance();

        Field optionsField = jspServlet.getClass().getDeclaredField("options");
        optionsField.setAccessible(true);
        EmbeddedServletOptions options = (EmbeddedServletOptions) optionsField.get(jspServlet);

        Field sysClassPathField = options.getClass().getDeclaredField("sysClassPath");
        sysClassPathField.setAccessible(true);

        String sysClassPath = (String) sysClassPathField.get(options);

        sysClassPathField.set(options, filterSysClassPath(sysClassPath));
    }

    private String filterSysClassPath(String sysClassPath) {
        if(sysClassPath == null) {
            return sysClassPath;
        } else {
            List<String> paths = new ArrayList<String>();
            for(String path : sysClassPath.split(File.pathSeparator)) {
                if(isJspCompilePath(path)) {
                    paths.add(path);
                }
            }
            StringBuilder newPath = new StringBuilder();

            for(String path : paths) {
                if(newPath.length() != 0) {
                    newPath.append(File.pathSeparator);
                }
                newPath.append(path);
            }

            return newPath.toString();
        }


    }

    private boolean isJspCompilePath(String path) {
        if(!path.contains(File.separator + "condi" + File.separator)) {
            return true;
        }

        String fileName = path.substring(path.lastIndexOf(File.separator)+1);
        if(fileName.startsWith("javax")) {
            return true;
        }
        if(fileName.contains("jasper")) {
            return true;
        }
        if(fileName.contains("org.apache.taglibs.standard")) {
            return true;
        }

        return false;
    }

}
